/**
 * Classe des points fixe dans l'interaction gravitationnelle
 *                   a trois corps restreinte
 *
 * Chaque objet correspond a un point fixe, etant donne le rapport de masse des deux corps pesants.
 *
 * Determination de la position a l'aide d'un algorithme de Newton-Raphson.
 *
 *  @param rapportDeMasse Masse reduite des deux corps pesants.
 *
 *  @author Benoit CARRY
 *  @version 05/12/2005
 */
public class PointFixe {

 //-Donnees
  private double q1,q2;         //Coordonnees cartesienne du point fixe
  private double[] paramEntree;  //Ensemble des parametres dont va depandre la determination des point fixes

   //-Mettre mu dans un tableau ne sert qu'a etre homogene avec les definitions de recherche de zero qui peuvent
   //- accepter en entree plus d'un parametre.
    //    parametres=new double[1];
    //    parametres[0]=mu;




   /**
    * Constructeur de la classe PointFixe
    *
    *  @param dependances Ensemble des parametres influant la determination des points fixes
    *
    */
  public PointFixe(double[] dependances) {paramEntree = dependances;}




   /**
    * Methode de recuperation des positions
    *
    *  @param choix Parametre indiquant si on desire la position en Q1 (choix ="q1") ou en Q2 (choix="q2")
    *  @return Renvoie la valeur de Q1 ou Q2 suivant le choix soumis
    */
  public double getPosition(String choix) {
    if ( "q1".equals(choix) ) {
      return q1;
    }
    else {
      return q2;
    }
  }






   /**
    * Methode de determination des points de Lagrange (points fixes).
    *
    *  @param numLagrange Numero du point de Lagrange desire
    *  @param precisionDuCalcul Precision sur la position desiree
    */
  public void determineLagrange(int numLagrange, double precisionDuCalcul) {

   /////////////////////////////////////////////////
   //--- Cas des 3 premiers points de Lagrange ---//
   /////////////////////////////////////////////////


    if ( numLagrange == 1 || numLagrange == 2 || numLagrange == 3 ) {

     //-Valeur de depart pour le calcul
      q2=0.0;   //Les 3 premiers points sont alignes avec les 2 corps pesants
      double[] zone= new double[2] ; //Intervalle contenant le zero
      String typeDeFonction;

     //-Choix du type de fonction suivant le point fixe considere
      if (numLagrange == 1) {typeDeFonction="Lagrange1";}
      else if (numLagrange == 2) {typeDeFonction="Lagrange2";}
      else {typeDeFonction="Lagrange3";}

     //-Creation des fonctions a calculer pour la determination du point a l'aide de la classe UsineAFonction.
      ChoixDeFonction foncL = UsineAFonction.creerFonc(typeDeFonction);
      ChoixDeFonction derivL = UsineAFonction.creerDer(typeDeFonction);

     //-Creation de la methode de determination du zero
      DetermineZero methode= new DetermineZero();


     //-Debbuging
      //System.out.println("Debut du calcul de la position du point de Lagrange L"+numLagrange);
     //-Debbuging



   ////////////////////////////////
   //--- Conditions initiales ---//
   ////////////////////////////////
      //-Proche des valeurs attendues


     //-Premier point de Lagrange :   -mu < q1 < 1-mu
      if ( numLagrange == 1 ) {
        zone[0]= -paramEntree[0] + 0.3 ;    //A 1/3
	zone[1]= 1.0 -paramEntree[0] - 0.3 ;  //A 2/3
      } 

     //-Second point de Lagrange :    1-mu < q1
      else if ( numLagrange == 2 ) {
        zone[0]=1.0-paramEntree[0]+  paramEntree[0]/10.0;
        zone[1]=1.0-paramEntree[0]+  2.0;
      }

     //-Troisieme point de Lagrange : q1 < -mu
      else {
        zone[0]=-1.0 -5.0*paramEntree[0]/12.0;
        zone[1]=-paramEntree[0]  ; //+0.5;
      }




   ///////////////////////////////////////////////////
   //--- Determination de l'intervalle de depart ---//
   ///////////////////////////////////////////////////

     //-Verifie que le zero est dans l'intervalle et augmente l'intervalle si necessaire
      zone=methode.verifZero( zone, paramEntree, foncL);

     //-Debbuging
      //System.out.println("Verifzone fini, l intervalle vaut : "+zone[0]+"\t"+zone[1]);
     //-Debbuging

     //-Calcul par Dichotomie de la pos du zero grossierement
      zone=methode.dichotomie( zone, 1e-1, paramEntree, foncL);

     //-Debbuging
      //System.out.println("Fin de Dichotomie");
      //System.out.println("L'intervalle vaut : [ "+zone[0]+" , "+zone[1]+" ]");
     //-Debbuging

     //-Calcul fin de la pos du 0 par NewtonRaphson
      q1=methode.newtonRaphson( zone[0], precisionDuCalcul, paramEntree, foncL, derivL);

    }



  //////////////////////////////////////////////
  //--- Cas du quatrieme point de Lagrange ---//
  //////////////////////////////////////////////

    if ( numLagrange == 4) {
      q1=0.5-paramEntree[0];
      q2=0.5*Math.sqrt(3);
    }


  //////////////////////////////////////////////
  //--- Cas du cinquieme point de Lagrange ---//
  //////////////////////////////////////////////

    if ( numLagrange == 5) {
      q1=0.5-paramEntree[0];
      q2=-0.5*Math.sqrt(3);
    }

  }



} //-Fin de la determination des points de Lagrange
