// interface graphique version deux pour le probleme a trois corps
// NB pour compilation javadoc : javadoc -author -link -version -d Documentation TroisCorps.java


import simu.*; 
import java.lang.Math;


/**
 * Algorithme de simulation pour SimuApplet.
 * Le problme  trois corps plan restreint (deux masses principales et une masse test).
 *
 * @author Aurelie GUILBERT  
 * @version 17/01/2006
 * <dl>
 * <dt>mu</dt>             <dd> parametre qui est utilise pour caracteriser les masses principales: soient m1 et m2 les deux masses en question, m1 est la plus grande des deux, alors, mu = m2 / (m1 + m2)</dd>
 * <dt>q1Initial</dt>      <dd> coordonnee 1 dedimensionnee relative a la position de la sonde, valeur initiale</dd>    
 * <dt>q2Initial</dt>      <dd> coordonnee 2 dedimensionnee relative a la position de la sonde, valeur initiale</dd>
 * <dt>q1dotInitial</dt>   <dd> coordonnee 1 dedimensionnee relative a la vitesse de la sonde, valeur initiale</dd>
 * <dt>q2dotInitial</dt>   <dd> coordonnee 2 dedimensionnee relative a la vitesse de la sonde, valeur initiale</dd>
 * <dt>p1Initial</dt>      <dd> moment conjugu a q1 et q2 au moment initial</dd>
 * <dt>p2Initial</dt>      <dd> moment conjugu a q1 et q2 au moment initial</dd>
 * <dt>deltaT</dt>         <dd> pas d'integration</dd>
 * <dt>dodo </dt>          <dd> nombre qui sert a la temporisation, doit absolument etre un entier</dd>
 *</dl>
 *
 * @see Integrateur
 * @see PointFixe
 *
 */
public class TroisCorps extends SimuApplet {

    Integrateur monIntegrateur;    // pour le calcul de la trajectoire   
    InterfaceEqMvmt mouvement;     // pour le calcul de l'integrale de Jacobi
    String fEqMvmt = "EqMvmtPblmTroisCorps";  // nom lie au nom du fichier contenant l'equation du mouvement

    double mu;
    double[] mutab;            // tableau de mu pour le calcul

    double q1Initial, q2Initial;          // position dans le repere tournant, initiale, suivant l'axe des masses
    double q1dotInitial, q2dotInitial;  // vitesses initiales 
    double p1Initial, p2Initial;        // moments conjugues a q1 et q2 au moment initial 

    double deltaT;             // pas d'integration
    int dodo;               // temporisation

    double[] voyager;          // tableau qui va ensuite servir pour le calcul: contient q1 q2 q1dot et q2dot

    int appel;                 // entier qui sert a construire la graphe en comptant les appels

    static  double jacobiInitial; // valeur de l'integrale de jacobi pour les conditions initiales

    String courbe;     // est ce qu'on veut tracer les courbes de vitesse nulle ?
    static double[] vitesse;


    // declaration des graphiques
    Affichage graphe = new Affichage();  // plot live dans repere tournant
    //Affichage fixe = new Affichage();    // plot live dans le repere fixe
    Affichage multi = new Affichage();   // multiplot avec Jacobi
    Affichage m1 = new Affichage();      // sous partie 1 du multiplot
    Affichage m2 = new Affichage();      // sous partie 2 du multiplot
    Affichage toufix = new Affichage();  // multiplot avec repere fixe
    Affichage tf1 = new Affichage();     // sous partie 1
    Affichage tf2 = new Affichage();     // sous partie 2
    Affichage courbeVitesse = new Affichage();     // courbe de vitesse nulle
    
 

    public TroisCorps() {


////////////////////////////////////////////////////////////////
//                     parametres d'entree                    //
////////////////////////////////////////////////////////////////


// ********* ---concernant les deux masses princiales...--- *********
// definition typique d'un parametre d'entree pour simuapplet

   ParamIn mu = new ParamIn();
   mu.type = "nombre";
   mu.label = "mu";
   mu.titre = "Mu";
   mu.acquisition = "champ";
   mu.defaut = "0.5";
   addParamIn(mu);


// ********* --- integration--- *********

   // pas d'integration
   ParamIn pas = new ParamIn();
   pas.type = "nombre";
   pas.label = "pasintegration";
   pas.titre = "Pas d'integration";
   pas.acquisition = "champ";
   pas.defaut = "0.001";
   addParamIn(pas);

   // temporisation
   ParamIn dodo = new ParamIn();
   dodo.type = "nombre";
   dodo.label = "ralentissement";
   dodo.titre = "Ralentissement";
   dodo.acquisition = "champ";
   dodo.defaut = "10";
   addParamIn(dodo);


// ********* ---concernant la sonde...--- *********

   // position initiale de la sonde, elle va etre de fait donnee dans le centre de masse
   ParamIn posx = new ParamIn();
   posx.type = "nombre";
   posx.label = "q1";
   posx.titre = "q1";
   posx.acquisition = "champ";
   posx.defaut = "0.0";
   addParamIn(posx);

   ParamIn posy = new ParamIn();
   posy.type = "nombre";
   posy.label = "q2";
   posy.titre = "q2";
   posy.acquisition = "champ";
   posy.defaut = "0.5";
   addParamIn(posy);

   // vitesse initiale de la sonde
   ParamIn vx = new ParamIn();
   vx.type = "nombre";
   vx.label = "q1point";
   vx.titre = "q1point";
   vx.acquisition = "champ";
   vx.defaut = "0";
   addParamIn(vx);

   ParamIn vy = new ParamIn();
   vy.type = "nombre";
   vy.label = "q2point";
   vy.titre = "q2point";
   vy.acquisition = "champ";
   vy.defaut = "0";
   addParamIn(vy);


// ********* ---ajustement automatique de la fenetre graphique--- *********

   ParamIn ajust = new ParamIn();
   ajust.type = "string";
   ajust.label = "ajustement";
   ajust.titre = "Ajustement automatique de la fentre graphique ?";
   ajust.acquisition = "case"; 
   ajust.defaut = "off";
   addParamIn(ajust);


// ********* ---voir les courbes de vitesse nulle ou pas ?--- *********

   ParamIn courbe = new ParamIn();
   courbe.type = "string";
   courbe.label = "vitessenulle";
   courbe.titre = "Voir juste les courbes de vitesse nulle ?";
   courbe.acquisition = "case"; 
   courbe.defaut = "off";
   addParamIn(courbe);
   

// ********* ---ajout d'une liste de valeur predefinies pour simplifier l'utilisation de cas typiques--- *********

   // trajectoire de henon
   ListeValeurs henon = new ListeValeurs();
   henon.ajouterDouble( "mu", 0.5 );
   henon.ajouterDouble("q1",  -0.267944);
   henon.ajouterDouble("q2", 0);
   henon.ajouterDouble("q1point", 0.0 );
   henon.ajouterDouble("q2point", 1.82134);
   henon.ajouterDouble("pasintegration", 0.05);
   henon.ajouterEntier("ralentissement", 100);
   addEnsemble("Trajectoire de Hnon", henon);

   // haltere 
   ListeValeurs jolie = new ListeValeurs();
   jolie.ajouterDouble( "mu", 0.02);
   jolie.ajouterDouble("q1", 0.48);
   jolie.ajouterDouble("q2", 0.966025);
   jolie.ajouterDouble("q1point", 0.5);
   jolie.ajouterDouble("q2point", -0.25);
   jolie.ajouterDouble("pasintegration", 0.05);
   jolie.ajouterEntier("ralentissement", 0);
   addEnsemble("Haltre", jolie );

   // trajectoire pres de L4
   ListeValeurs pres = new ListeValeurs();
   pres.ajouterDouble("mu", 0.02);
   pres.ajouterDouble("q1", 0.5);
   pres.ajouterDouble("q2", 0.866025);
   pres.ajouterDouble("q1point", 0.0);
   pres.ajouterDouble("q2opint", -0.02);
   pres.ajouterDouble("pasintegration", 0.5);
   pres.ajouterEntier("ralentissement", 0);
   addEnsemble("Trajectoire autour de L4", pres);

   // banane
   ListeValeurs banane = new ListeValeurs();
   banane.ajouterDouble( "mu", 0.0009547861);
   banane.ajouterDouble("q1", 0.525000000);
   banane.ajouterDouble("q2", -0.909326674);
   banane.ajouterDouble("q1point", -0.064718610);
   banane.ajouterDouble("q2point", -0.044371494);
   banane.ajouterDouble("pasintegration", 0.5);
   banane.ajouterEntier("ralentissement", 0);
   addEnsemble("Trajectoire autour de L5", banane);

   // fer a cheval 4
   ListeValeurs horse4 = new ListeValeurs();
   horse4.ajouterDouble( "mu", 0.0009547861);
   horse4.ajouterDouble("q1", 0.520000000);
   horse4.ajouterDouble("q2", 0.900666420);
   horse4.ajouterDouble("q1point", 0.051579626);
   horse4.ajouterDouble("q2point", -0.029668175);
   horse4.ajouterDouble("pasintegration", 1.);
   horse4.ajouterEntier("ralentissement", 0);
   addEnsemble("Fer  cheval 1", horse4);

   // fer a cheval 6
   ListeValeurs horse6 = new ListeValeurs();
   horse6.ajouterDouble( "mu", 0.0009547861);
   horse6.ajouterDouble("q1", 0.53);
   horse6.ajouterDouble("q2", 0.917986928);
   horse6.ajouterDouble("q1point", 0.077336855);
   horse6.ajouterDouble("q2point", -0.044371494);
   horse6.ajouterDouble("pasintegration", 0.3);
   horse6.ajouterEntier("ralentissement", 10);
   addEnsemble("Fer a cheval 2", horse6);

   // crash sur Jupiter 1
   //ListeValeurs crash1 = new ListeValeurs();
   //crash1.ajouter( "mu", 0.0009547861);
   //crash1.ajouter("x", 0.5);
   //crash1.ajouter("y", 0.86);
   //crash1.ajouter("vx", 0.0);
   //crash1.ajouter("vy", -0.1);
   //crash1.ajouter("pas", 0.3);
   //crash1.ajouter("ralentissement", 0);
   //addEnsemble("Crash sur Jupiter", crash1);
   // mis en commentaire parce qu'il se trouve qu'avec notre intgrateur, la sonde ne se crache pas !

   // tube 2
   ListeValeurs tub2 = new ListeValeurs();
   tub2.ajouterDouble( "mu", 0.0009547861);
   tub2.ajouterDouble("q1", 0.495);
   tub2.ajouterDouble("q2", 0.874685658);
   tub2.ajouterDouble("q1point", 0.012987562);
   tub2.ajouterDouble("q2point", 0.007488536);
   tub2.ajouterDouble("pasintegration", 0.5);
   tub2.ajouterEntier("ralentissement", 0);
   addEnsemble("Tube autour de L4", tub2);

   // helio
   ListeValeurs helio = new ListeValeurs();
   helio.ajouterDouble( "mu", 0.0009547861);
   helio.ajouterDouble("q1", 0.530000000);
   helio.ajouterDouble("q2", 0.082013072);
   helio.ajouterDouble("q1point", 0.077336855);
   helio.ajouterDouble("q2point", -0.044371494);
   helio.ajouterDouble("pasintegration", 0.01);
   helio.ajouterEntier("ralentissement", 0);
   addEnsemble("Trajectoire heliocentrique", helio);

   // triskel
   ListeValeurs triskel = new ListeValeurs();
   triskel.ajouterDouble( "mu", 0.0009547861);
   triskel.ajouterDouble("q1", 0.5);
   triskel.ajouterDouble("q2", 0.866025403784439);
   triskel.ajouterDouble("q1point", -0.1);
   triskel.ajouterDouble("q2point", 0.1);
   triskel.ajouterDouble("pasintegration", 0.1);
   triskel.ajouterEntier("ralentissement", 0);
   addEnsemble("Triskel", triskel);




//////////////////////////////////////////////////////////////////
//                    parametres de sortie	             	//
//////////////////////////////////////////////////////////////////


   // pour le trac de la trajectoire dans le repere tournant
   ParamOut q1 = new ParamOut();
   q1.type = "nombre";
   q1.label = "q1";
   addParamOut(q1);

   ParamOut q2 = new ParamOut();
   q2.type = "nombre";
   q2.label = "q2";
   addParamOut(q2);



   // Pour tracer la valeur relative de l'integrale de Jacobi
   ParamOut jacobiRelatif = new ParamOut();
   jacobiRelatif.type = "nombre";
   jacobiRelatif.label = "jacobiRelatif";
   addParamOut(jacobiRelatif);
 
   ParamOut appel = new ParamOut();
   appel.type = "nombre";
   appel.label = "appel";
   addParamOut(appel);



   // Pour tracer le trajectoire dans le repere fixe
   ParamOut q1fixe = new ParamOut();
   q1fixe.type = "nombre";
   q1fixe.label = "q1fixe";
   addParamOut(q1fixe);

   ParamOut q2fixe = new ParamOut();
   q2fixe.type = "nombre";
   q2fixe.label = "q2fixe";
   addParamOut(q2fixe);



   // Pour donner la valeur de Jacobi initial
   ParamOut info = new ParamOut();
   info.type = "string";
   info.label = "jacobiInitial";
   addParamOut(info);






/////////////////////////////////////////////////////////////
//                   les graphiques                        //
/////////////////////////////////////////////////////////////


// pour la realisation du graphe, certains parametres vont etre modifies au cours du programme, ar exemle la connection des points et le style de points, ceci dans le but de differencier les masses, les points de Lagrange, la trajectoire de la sonde.



////////////////////////////////////////////////////////////////////////
//     plot live qui donne la trajectoire dans le repere tournant     //
////////////////////////////////////////////////////////////////////////

// le nom de ce plot est "graphe", a retenir pour modifier ensuite certains parametres

   graphe.type = "plotlive";
   graphe.titre = "Repre tournant";
   String[][] pparam = {{"q1", "q2"}};
   graphe.params = pparam;
   graphe.carre = true;
 
   // initialisation de la connection, du style de points: pour les masses et les points de Lagrange
   graphe.points = "dots";
   graphe.connecter = false;
   
   // ajustement automatique de la fenetre graphique
   //graphe.ajuster = true;
   addAffichage(graphe);





////////////////////////////////////////////////////////////////////////
//     plot live qui donne la trajectoire dans le repere fixe         //
////////////////////////////////////////////////////////////////////////

// le nom de ce plot est "fixe", a retenir pour modifier ensuite certains parametres
/*
   fixe.type = "plotlive";
   fixe.titre = "Trajectoire live dans le repre fixe";
   String[][] pparamf = {{"q1fixe", "q2fixe"}};
   fixe.params = pparamf;
   fixe.carre = true;
 
   // initialisation de la connection, du style de points: pour les masses et les points de Lagrange
   fixe.points = "none";
   fixe.connecter = false;

   // ajustement automatique de la fenetre graphique
   //fixe.ajuster = true;
   addAffichage(fixe);
*/



///////////////////////////////////////////////////////////////////////
//            affichage des courbes de vitesse nulle                 //
///////////////////////////////////////////////////////////////////////

   //   Affichage courbeVitesse = nex Affichage();
   courbeVitesse.type = "plotlive";
   courbeVitesse.titre = "Courbe de vitesse nulle";
   String[][] pparamc = {{"q1", "q2"}};
   courbeVitesse.params = pparamc;
   courbeVitesse.carre = true;

   // initialisation de la connection des points : pour les masses et les points de Lagrange
   courbeVitesse.points = "dots";
   courbeVitesse.connecter = false;

   // ajustement automatique de la fenetre graphique
   //courbeVitesse.ajuster = true;
   addAffichage(courbeVitesse);




///////////////////////////////////////////////////////////////////////
//          affichage multi-plot trajectoire et jacobi               //
///////////////////////////////////////////////////////////////////////


   //   Affichage multi = new Affichage();
   multi.type = "multiplot";
   multi.titre = "Trajectoire et integrale de Jacobi";
   multi.sousAff = new Affichage[2];


   // le nom du graphe qui concerne la trajectoire est m1, pour la modification des parametres
   //   Affichage m1 = new Affichage();
   m1.type = "plotlive";
   m1.titre = "Trajectoire";
   String[][] pparamsm1 = {{"q1", "q2"}};
   m1.params = pparamsm1;
   m1.carre = true;
   multi.sousAff[0] = m1;

   // initialisation de la connection et du style de points: masses et points de Lagrange
   m1.connecter = false;
   m1.points = "dots";

   // ajustement automatique de la fenetre graphique
   //m1.ajuster = true;


   // le nom du graphe qui concerne l'integrale de Jacobi est m2, les parametres ne doivent pas changer.
   //   Affichage m2 = new Affichage();
   m2.type = "plotlive";
   m2.titre = "Valeur relative de l'integrale de Jacobi";
   String[][] pparamsm2 = {{"appel","jacobiRelatif"}};
   m2.params = pparamsm2;
   multi.sousAff[1] = m2;


   // initialisation de la connection et du style de points
   m2.connecter = true;
   m2.points = "points";

   // ajustement automatique de la fenetre graphique
   //m2.ajuster = true;


   addAffichage(multi);




///////////////////////////////////////////////////////////////////
//         affichage multi-plot repere tournant et fixe          //
///////////////////////////////////////////////////////////////////


   //   Affichage toufix = new Affichage();
   toufix.type = "multiplot";
   toufix.titre = "Repres tournant et fixe";
   toufix.sousAff = new Affichage[2];


   // le nom du plot qui concerne la trajectoire dans le repere tournant est tf1, pour la modification des parammetres
   tf1.type = "plotlive";
   tf1.titre = "Dans le repere tournant";
   String[][] pparamstf1 = {{"q1", "q2"}};
   tf1.params = pparamstf1;
   tf1.carre = true;    
   toufix.sousAff[0] = tf1;

   // initialisation de la connection et du style de points, masses et points de Lagrange
   tf1.connecter = false;
   tf1.points = "dots";

   // ajustement automatique de la fenetre graphique
   //tf1.ajuster = true;


   // le nom du plot qui concerne la trajectoire dans le repere fixe est tf2
   tf2.type = "plotlive";
   tf2.titre = "Dans le repere fixe";
   String[][] pparamstf2 = {{"q1fixe", "q2fixe"}};
   tf2.params = pparamstf2;
   tf2.carre = true;
   toufix.sousAff[1] = tf2;

   // attention, initialisation diffrente car pour l'instant, on ne trace pas les masses et les points de Lagrange !
   // initialisation de la connection et du style de points
   tf2.connecter = false ;
   tf2.points = "none";

   // ajustement automatique de la fenetre graphique
   //tf2.ajuster = true;

   addAffichage(toufix);


   appletDoc("doc_applet.xml");
 }



///////////////////////////////////////////////////////////
//                       calcul live                     //
///////////////////////////////////////////////////////////




// ********* ---initialisation--- *********

public void initCalculLive(ListeValeurs in) throws SimuException {

    // ajustement automatique des fenetres graphiques
    String ajust = in.lireString("ajustement");
    if (ajust.equals("on")) {
	graphe.ajuster = true;
        //fixe.ajuster = true;
        m1.ajuster = true;
        m2.ajuster = true;
        tf1.ajuster = true;
        tf2.ajuster = true;
        courbeVitesse.ajuster = true;
    }
    else {
	graphe.ajuster = false;
        //fixe.ajuster = false;
        m1.ajuster = false;
        m2.ajuster = true;
        tf1.ajuster = false;
        tf2.ajuster = false;
        courbeVitesse.ajuster = false;
    }


    // courbe de vitesse nulle ?
    courbe = in.lireString("vitessenulle");




    // initialisation des diverses valeurs necessaires au calcul
   q1Initial = in.lireDouble("q1");
   q2Initial = in.lireDouble("q2");

   q1dotInitial = in.lireDouble("q1point");
   q2dotInitial = in.lireDouble("q2point");

   p1Initial = q1dotInitial - q2Initial;
   p2Initial = q2dotInitial + q1Initial;

   // pas d'integration
   deltaT = in.lireDouble("pasintegration");

   // temporisation
   dodo = in.lireEntier("ralentissement");
   
   // valeur du mu
   mu = in.lireDouble("mu");




   // tableau transmis au calcul pour l'initialisation, contient q1 q2 q1dot q2dot
   double[] initialisation = new double[4];
   initialisation[0] = q1Initial;
   initialisation[1] = q2Initial;
   initialisation[2] = p1Initial;
   initialisation[3] = p2Initial;
   voyager = initialisation;


   // mu sous forme de tableau, envoye au calcul, pour une hstoire de generalisation il a besoin d'un tableau
   mutab = new double[1];
   mutab[0] = mu;



   // ********* ---pour le calcul de la trajectoire--- *********
   //monIntegrateur = new Integrateur(4,mutab,"EqMvmtPblmTroisCorps");
   monIntegrateur = new Integrateur(4,mutab,fEqMvmt);



   // ********* ---pour la calcul de l'integrale de Jacobi--- *********
   mouvement = UsineEqMvmt.nouveau(fEqMvmt);
   jacobiInitial = mouvement.intPrem(voyager,mutab);



   // ********* ---courbe de vitesse nulle--- *********
   CourbeVitesseNulle vitesseNulle = new CourbeVitesseNulle(mu);
   vitesse =  vitesseNulle.calculCourbeV0(jacobiInitial, 1e-3);



   // initialisation du compteur
   appel = 0;


   // changement de style de points: la c'est pour quand on joue plusieurs fois a relancer l'applet, pour reinitialiser correctement !
   graphe.points = "dots";
   //fixe.points = "none";
   m1.points = "dots";
   m2.points = "points";
   tf1.points = "dots";
   tf2.points = "none";
   courbeVitesse.points = "dots";

 }


// ********* ---apres l'initialisation, le calcul de la trajectoire--- *********


public ListeValeurs calculLive() throws SimuException {

    // indication de la precision
    double precision = 1.0e-6;


   // tableau qui va contenir tous les parametres de sortie   
   ListeValeurs out = new ListeValeurs();


   PointFixe pointDeLagrange;   // pour les points de Lagrange
   double q1, q2;         // parametres de sortie donnes dans le repere tournant
   double p1, p2;         // moments conjugues 
   double q1dot, q2dot;   // vitesses de sortie dans le repere tournant
   double q1fixe, q2fixe;  // parametres de sortie pour le repere fixe
   double jacobi;  // integrale de jacobi absolue
   double jacobiRelatif;  // valeur relative de l'intergrale de Jacobi





   // on a maintenant deux cas : soit c'est la courbe de vitesse nulle qui est demande et dans ce cas, on va  "else", soit elle n'est pas demandee, et on fait ce qui suit...


    //////////////////////////////////////////////////////////////
    //      si la courbe de vitesse nulle n'est pas demandee    //
    //////////////////////////////////////////////////////////////

   if (courbe.equals("off")){

   /////////////////////////////////////////////////////////////////////////
   // pour tracer les graphes et faire le calcul:                         //
   // * appel 1 = position de la masse 1                                  //
   // * appel 2 = position de la masse 2                                  //
   // * pour les 5 appels suivants (2 < appel < 8 ): points de lagrange   //
   // * appel == 8: position initiale de la sonde                         //
   // * appel == 9: initialisation du calcul de trajectoire               //
   // * appel > 9: reste de la trajectoire                                //
   /////////////////////////////////////////////////////////////////////////


   // le compteur evolue avec le nombre d'appel du calcul
   appel++;



   // ********* ---premiere masse--- *********

   if (appel == 1) {
       // pour la trajectoie dans le repere tournant
       q1 = - mu;
       q2 = 0;

       // pour la trajectoire dans le repere fixe
       q1fixe = q1*Math.cos(0.) - q2*Math.sin(0.);
       q2fixe = q1*Math.sin(0.) + q2*Math.cos(0.); 

       // pour la calcul de l'integrale de Jacobi
       jacobi = mouvement.intPrem(voyager,mutab);
       jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;

           
       // connection entre les points
       graphe.connecter = false;
       //fixe.connecter = false;
       m1.connecter = false;
       m2.connecter = true;
       tf1.connecter = false;
       tf2.connecter = false;
       courbeVitesse.connecter = false;

   }

   // ********* ---seconde masse--- *********

   else if (appel == 2) {
       // pour la trajectoire dans le repere tournant
       q1 = 1.0 - mu;
       q2 = 0;

       // pour la trajectoire dans le repere fixe
       q1fixe = q1*Math.cos(0.) - q2*Math.sin(0.);
       q2fixe = q1*Math.sin(0.) + q2*Math.cos(0.);

       // pour l'integrale de Jacobi
       jacobi = mouvement.intPrem(voyager,mutab);
       jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;

       // pour changer la couleur des points, entre les masses et les points de Lagrange
       out.ajouterString("fin", "courbe");    
   } 


   // ********* --- cinq points de lagrange--- *********

   else if (2 < appel && appel < 8) {

       pointDeLagrange = new PointFixe(mutab);
       int numeroDeLagrange = appel - 2;
       pointDeLagrange.determineLagrange(numeroDeLagrange, 1e-13);

       // pour le repere tournant	   
       q1 = pointDeLagrange.getPosition("q1");
       q2 = pointDeLagrange.getPosition("q2");
	   
       // pour le repere fixe
       q1fixe = q1*Math.cos(0.) - q2*Math.sin(0.);
       q2fixe = q1*Math.sin(0.) + q2*Math.cos(0.);

       // pour l'integrale de Jacobi
       jacobi = mouvement.intPrem(voyager,mutab);
       jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;


       if (appel == 7) { 
           // change de nouveau la couleur des points, entre les points de Lagrange et la trajectoire
           out.ajouterString("fin", "courbe");

	   // changement de caracteristiques pour les plots
	   // pour passer  la trajectoire dans les divers graphes
           graphe.connecter = true;
           graphe.points = "points";
           //fixe.connecter = true;
           //fixe.points = "points";
           m1.connecter = true;
           m1.points = "points";
           tf1.connecter = true;
           tf1.points = "points";
           tf2.connecter = true;
           tf2.points = "points";
           courbeVitesse.connecter = true;
           courbeVitesse.points = "points";

       }
     }


   // ********* ---position initiale de la sonde--- *********

   else if (appel == 8) {
       // pour le repere tournant
       q1 = q1Initial;
       q2 = q2Initial;

       // pour l'integrale de Jacobi
       jacobi = mouvement.intPrem(voyager,mutab);
       jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;

       // pour le repere fixe
       q1fixe = q1*Math.cos(appel*deltaT) - q2*Math.sin(appel*deltaT);
       q2fixe = q1*Math.sin(appel*deltaT) + q2*Math.cos(appel*deltaT);

   }


   // ********* ---premier appel de l'integrateur: premier point trace--- *********

   else if (appel == 9) {
       voyager = monIntegrateur.rk4pv(voyager, deltaT, precision); 

       // dans le repre tournant	     
       q1 = voyager[0];
       q2 = voyager[1];
       //p1 = voyager[2];
       //p2 = voyager[3];
	   	   
       // dans le repere fixe
       q1fixe = q1*Math.cos(appel*deltaT) - q2*Math.sin(appel*deltaT);
       q2fixe = q1*Math.sin(appel*deltaT) + q2*Math.cos(appel*deltaT);

       // calcul de l'integrale de jacobi
       jacobi = mouvement.intPrem(voyager,mutab);
       jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;

   }

   // ********* ---toutes les autres positions de la sonde--- *********

   else {
      // temporisation
      dormir(dodo);

	   
       // *** ---debuggaing--- ***
       //System.out.println("coucou");
       // *** ---debugging--- ***

	   
       voyager = monIntegrateur.rk4pv(voyager, deltaT, precision, true);

	   
       // *** ---debugging--- ***
       //System.out.println("hello");
       // *** ---debugging--- ***


       // pour le repere tournant	   
       q1 = voyager[0];
       q2 = voyager[1];
       //p1 = voyager[2];
       //p2 = voyager[3];
	   
       // pour le repere fixe
       q1fixe = q1*Math.cos(appel*deltaT) - q2*Math.sin(appel*deltaT);
       q2fixe = q1*Math.sin(appel*deltaT) + q2*Math.cos(appel*deltaT);

       // calcul de l'integrale de Jacobi
       jacobi = mouvement.intPrem(voyager,mutab);
       jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;

   }



// ********* ---remplissage du tableau de parametres de sortie---*********

    // pour la trajectoire dans le repere tournant
    out.ajouterDouble("q1", q1);
    out.ajouterDouble("q2", q2);
    // pour l'integrale de Jacobi
    out.ajouterDouble("jacobiRelatif", jacobiRelatif);
    out.ajouterDouble("appel", appel);
    // pour la trajectoire dans le repere fixe
    out.ajouterDouble("q1fixe", q1fixe);
    out.ajouterDouble("q2fixe", q2fixe);

    out.ajouterString("jacobiInitial", "valeur initiale de l'intgrale de Jacobi " + jacobiInitial);

    return out;
    }








    ///////////////////////////////////////////////////////////
    //      si la courbe de vitesse nulle est demandee       //
    ///////////////////////////////////////////////////////////

    else {

   // le compteur evolue avec le nombre d'appel du calcul
   appel++;


   //////////////////////////////////////////////////
   // appel 1: premiere masse                      //
   // appel 2: seconde masse                       //
   // appels 3 a 7: points de Lagrange             //
   // reste des appels: courbe de vitesse nulle    //
   //////////////////////////////////////////////////



   // ********* ---premiere masse--- *********

   if (appel == 1) {
       // pour la trajectoie dans le repere tournant
       q1 = - mu;
       q2 = 0;

       // pour la trajectoire dans le repere fixe
       q1fixe = q1*Math.cos(0.) - q2*Math.sin(0.);
       q2fixe = q1*Math.sin(0.) + q2*Math.cos(0.); 

       // pour la calcul de l'integrale de Jacobi
       jacobi = mouvement.intPrem(voyager,mutab);
       jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;

           
       // connection entre les points
       graphe.connecter = false;
       //fixe.connecter = false;
       m1.connecter = false;
       m2.connecter = true;
       tf1.connecter = false;
       tf2.connecter = false;
       courbeVitesse.connecter = false;

   }

   // ********* ---seconde masse--- *********

   else if (appel == 2) {
       // pour la trajectoire dans le repere tournant
       q1 = 1.0 - mu;
       q2 = 0;

       // pour la trajectoire dans le repere fixe
       q1fixe = q1*Math.cos(0.) - q2*Math.sin(0.);
       q2fixe = q1*Math.sin(0.) + q2*Math.cos(0.);

       // pour l'integrale de Jacobi
       jacobi = mouvement.intPrem(voyager,mutab);
       jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;

       // pour changer la couleur des points, entre les masses et les points de Lagrange
       out.ajouterString("fin", "courbe");    
   } 


   // ********* --- cinq points de lagrange--- *********

   else if (2 < appel && appel < 8) {

       pointDeLagrange = new PointFixe(mutab);
       int numeroDeLagrange = appel - 2;
       pointDeLagrange.determineLagrange(numeroDeLagrange, 1e-13);

       // pour le repere tournant	   
       q1 = pointDeLagrange.getPosition("q1");
       q2 = pointDeLagrange.getPosition("q2");
	   
       // pour le repere fixe
       q1fixe = q1*Math.cos(0.) - q2*Math.sin(0.);
       q2fixe = q1*Math.sin(0.) + q2*Math.cos(0.);

       // pour l'integrale de Jacobi
       jacobi = mouvement.intPrem(voyager,mutab);
       jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;


       if (appel == 7) { 
           // change de nouveau la couleur des points, entre les points de Lagrange et la trajectoire
           out.ajouterString("fin", "courbe");

	   // changement de caracteristiques pour les plots
	   // pour passer  la trajectoire dans les divers graphes
           graphe.connecter = true;
           graphe.points = "points";
           //fixe.connecter = true;
           //fixe.points = "points";
           m1.connecter = true;
           m1.points = "points";
           tf1.connecter = true;
           tf1.points = "points";
           tf2.connecter = true;
           tf2.points = "points";
           courbeVitesse.connecter = false;
           courbeVitesse.points = "points";
       }
     }
   else {
       // temporisation, enleve parce que pour la courbe de vitesse nulle c'est vraiment trop lent
       // dormir(dodo); 

       int n = (int) vitesse[0];


       // *** ---debugging--- ***
       // System.out.println("n =" + n);
       // System.out.println("BOUCLE POUR AFFICHER");
       // *** ---debugging--- ***


       //si n = 0, alors les q1 et q2 n'existent as pour le reste des iterations, donc on donne des valeurs de q1 et q2 qui correspondent  une des deux masses... pour qu'il ne mette pas un point n'importe ou...
       if (n == 0){
	   q1 = 0.0;
           q2 = 0.0;
       }

       else {

         // *** ---debugging--- ***
         // System.out.println("n =" + n);
         // *** ---debugging--- ***

       
         q1 = vitesse[appel - 7];
         q2 = vitesse[appel - 7 + n];


         // *** ---debugging--- ***
         // System.out.println("q1 = " + q1+"\t"+"q2 = " + q2);
         // *** ---debugging--- ***

       }

       /*
       for (int i = 1; i < n+1 ; i++){
	   q1 = vitesse[i];
           q2 = vitesse[i+n];
       }
       */


       q1fixe = 0.0;
       q2fixe = 0.0;

       jacobi = mouvement.intPrem(voyager,mutab);
       jacobiRelatif = 0.0;
            
       if (appel == n + 8){
	   out.ajouterString("fin", "simulation");
          }
    }


   // remplissage du tableau de parametres de sortie
   // pour la position de la sonde
    out.ajouterDouble("q1", q1);
    out.ajouterDouble("q2", q2);
    // pour l'integrale de Jacobi
    out.ajouterDouble("jacobiRelatif", jacobiRelatif);
    out.ajouterDouble("appel", appel);
    // pour la position de la sonde dans le repere fixe
    out.ajouterDouble("q1fixe", q1fixe);
    out.ajouterDouble("q2fixe", q2fixe);
    // pour avoir une ide de la valeur initiale de l'integrale de Jacobi
    out.ajouterString("jacobiInitial", "valeur initiale de l'intgrale de Jacobi " + jacobiInitial);

    return out;
   }

 }
}
